package com.gic.store.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.dto.ClerkSearchDTO;
import com.gic.store.dto.ClerkStoreListDTO;
import com.gic.store.dto.clerk.ProcessBatchClerkDTO;

import java.util.List;

/**
 * @ClassName: ClerkApiService
 * @Description: 导购service 
 * @author zhiwj
 * @date 2019/7/31 11:36
 */
public interface ClerkApiService {
    
    /**
     * @Title: saveOrUpdate
     * @Description: 保存或编辑导购
     * @author zhiwj 
     * @param clerkDTO 导购
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws
     */
    ServiceResponse<Void> saveOrUpdate(ClerkDTO clerkDTO);

    /** @Description: 订单途经保存导购
     * @author taogs
     * @Date 19:52 2020/1/8
     * @Param 
     * @return 
    */
    ServiceResponse<Integer> saveClerkForOrder(String clerkName, String clerkCode, Integer storeId, Integer enterpriseId, String orderNum);

    
    /** @Description: 通过导购code查询导购
     * @author taogs
     * @Date 20:04 2020/1/8
     * @Param 
     * @return 
    */
    ServiceResponse<ClerkDTO> getClerkByCode(Integer enterpriseId, String clerkCode, Integer storeId);
    /**
     * @Title: listClerk
     * @Description: 导购列表
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param clerkSearchDTO
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.store.dto.ClerkStoreListDTO>>
     * @throws 
     */
    ServiceResponse<Page<ClerkStoreListDTO>> listClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO, Integer pageNum, Integer pageSize);

    /**
     * 获取clerk总数
     * @Title: getTotalClerk
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkSearchDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> getTotalClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO);

    /**
     * 批量修改状态
     * @Title: updateClerkStatus
     * @Description: 批量修改导购状态
     * @author zhiwj 
     * @param enterpriseId 企业id
     * @param clerkIds 导购ids
     * @param status 状态
     * @return com.gic.api.base.commons.ServiceResponse
     * @throws 
     */
    ServiceResponse<Void> updateClerkStatus(Integer enterpriseId, String clerkIds, Integer status);

    /**
     * 查询list通过id
     * @Title: listClerkByIds
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param clerkIds
     * @return java.util.List<com.gic.store.dto.ClerkDTO>
     * @throws
     */
    ServiceResponse<List<ClerkDTO>> listClerkByIds(Integer enterpriseId, String clerkIds);

    ServiceResponse<List<ClerkDTO>> listClerkByStoreId(Integer enterpriseId, Integer storeId, String search);

    ServiceResponse<ClerkDTO> getClerkById(Integer enterpriseId, Integer clerkId);

    ServiceResponse<List<ClerkDTO>> listClerk(Integer enterpriseId, String search);

    /**
     * 处理导购 实施运维调用批处理
     * @Title: importClerkForOperation 
     * @Description:
     * @author guojuxing
     * @param clerkInfo 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> importClerkForOperation(ProcessBatchClerkDTO clerkInfo);

    /**
     * @Title: addClerkToOtherStore
     * @Description: 将同样导购的信息添加到另一个门店下
     *  入参 storeId clerkId enterpriseId  判断该导购是否与该门店存在关联，
     *  如果存在返回 门店下 同导购code 的 clerkId ,如果不存在，新建导购，返回新建的导购i
     * @author zhiwj
     * @param clerkId
     * @param storeId
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Integer> addClerkToOtherStore(Integer clerkId, Integer storeId, Integer enterpriseId);

    /**
     * @Title: listClerkByStoreBrand
     * @Description: 按店招品牌搜索所有导购
     * @author zhiwj
     * @param enterpriseId
     * @param storeBrandId
     * @param clerkName
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.ClerkDTO>>
     * @throws
     */
    ServiceResponse<List<ClerkDTO>> listClerkByStoreBrand(Integer enterpriseId, Integer storeBrandId, String clerkName);

}
