package com.gic.store.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.store.dto.StoreDictDTO;

import java.util.List;

public interface StoreDictApiService {

    /**
     * 查询运维后台所有经营类目
     * @Title: listAllBusinessCategory
     * @Description:
     * @author zhiwj
     *  @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreDictDTO>>
     * @throws 
     */
    ServiceResponse<List<StoreDictDTO>> listAllBusinessCategory(Integer enterpriseId);

    /**
     *
     * @Title: getStoreDict
     * @Description:
     * @author zhiwj
     * @param categoryCode
     * @param code
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.store.dto.StoreDictDTO>
     * @throws
     */
    ServiceResponse<StoreDictDTO> getStoreDict(String categoryCode, String code);

    /**
     * 查询运维后台所有门店类型，企业选中checked=1
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<StoreDictDTO>> listAllStoreType(Integer enterpriseId);

    /**
     * 查询运维后台所有门店状态，企业选中checked=1
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<StoreDictDTO>> listAllStoreStatus(Integer enterpriseId);

    /**
     * 查询企业下门店类型
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<StoreDictDTO>> listStoreType(Integer enterpriseId);

    /**
     * 查询企业下门店状态
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<StoreDictDTO>> listStoreStatus(Integer enterpriseId);

    /**
     * 保存门店类型
     * @param enterpriseId
     * @param storeTypeArr
     * @return
     */
    ServiceResponse saveStoreType(Integer enterpriseId, String[] storeTypeArr);

    /**
     * 保存门店类型
     * @param enterpriseId
     * @param storeStatusArr
     * @return
     */
    ServiceResponse saveStoreStatus(Integer enterpriseId, String[] storeStatusArr);

    ServiceResponse saveStoreStatusAndStoreType(Integer enterpriseId, String[] storeStatusArr, String[] storeTypeArr);

    /**
     * 查询支持列表查询的自定义字段
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<StoreDictDTO>> listStoreField(Integer enterpriseId);

    /**
     * 查询支持列表查询的自定义字段：过滤文本类型自定义字段
     * @Title: listStoreFieldOfFilterText 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.store.dto.StoreDictDTO>> 
     */
    ServiceResponse<List<StoreDictDTO>> listStoreFieldOfFilterText(Integer enterpriseId);

    /**
     * 保存自定义字段关联查询
     * @param enterpriseId
     * @param value
     * @return
     */
    ServiceResponse saveStoreField(Integer enterpriseId, String value, int type, Integer userId);


    /** @Description: 列表筛选字段配置
     * @author taogs
     * @Date 18:16 2020/3/11
     * @Param 
     * @return 
    */
    ServiceResponse saveStoreField(Integer enterpriseId, String values, Integer userId);
    /**
     * 查询列表/筛选属性光支持 列表数据
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<String>> listStoreFieldDictValue(Integer enterpriseId);
}