package com.gic.store.service.impl;

import com.gic.commons.util.GlobalInfo;
import com.gic.store.constant.ClerkGenderEnum;
import com.gic.store.constant.ClerkLogTypeEnum;
import com.gic.store.dao.mapper.TabClerkLogMapper;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.dto.ClerkLogDTO;
import com.gic.store.entity.TabClerk;
import com.gic.store.entity.TabClerkLog;
import com.gic.store.service.ClerkLogService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-11 9:43
 */
@Service
public class ClerkLogServiceImpl implements ClerkLogService {

    @Autowired
    private TabClerkLogMapper tabClerkLogMapper;

    @Override
    public Integer saveClerkLogBySaveClerk(Integer enterpriseId, Integer clerkId, Integer reason, String remark, Integer operatorId, String operatorName) {
        TabClerkLog clerkLog = new TabClerkLog();
        clerkLog.setEnterpriseId(enterpriseId);
        clerkLog.setClerkId(clerkId);
        clerkLog.setContent("导购新增");
        clerkLog.setReason(reason);
        clerkLog.setLogType(ClerkLogTypeEnum.SAVE.getCode());
        clerkLog.setRemark(remark);
        clerkLog.setCreateTime(new Date());
        clerkLog.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        clerkLog.setOperatorId(operatorId);
        clerkLog.setOperatorName(operatorName);
        return tabClerkLogMapper.insertSelective(clerkLog);
    }

    @Override
    public Integer saveClerkLogByUpdateClerk(TabClerk oldClerk, ClerkDTO newClerk, Integer enterpriseId, Integer clerkId, Integer reason, String remark, Integer operatorId, String operatorName) {
        TabClerkLog clerkLog = new TabClerkLog();
        clerkLog.setEnterpriseId(enterpriseId);
        clerkLog.setClerkId(clerkId);
        clerkLog.setReason(reason);
        clerkLog.setLogType(ClerkLogTypeEnum.UPDATE.getCode());
        clerkLog.setRemark(remark);
        clerkLog.setCreateTime(new Date());
        clerkLog.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        clerkLog.setOperatorId(operatorId);
        clerkLog.setOperatorName(operatorName);

        StringBuilder sb = new StringBuilder();
        if (StringUtils.isNotBlank(newClerk.getClerkName()) && !StringUtils.equals(newClerk.getClerkName(), oldClerk.getClerkName())) {
            sb.append("导购名称").append("【").append(oldClerk.getClerkName()).append("】")
                    .append("变更为【").append(newClerk.getClerkName()).append("】");
        }
        if (newClerk.getClerkGender() != null && !newClerk.getClerkGender().equals(oldClerk.getClerkGender())) {
            sb.append("性别").append("【").append(ClerkGenderEnum.parseCode(oldClerk.getClerkGender())).append("】")
                    .append("变更为【").append(ClerkGenderEnum.parseCode(newClerk.getClerkGender())).append("】");
        }
        if (StringUtils.isNotBlank(newClerk.getNationcode()) && !StringUtils.equals(newClerk.getNationcode(), oldClerk.getNationcode())) {
            sb.append("区号").append("【").append(oldClerk.getNationcode()).append("】")
                    .append("变更为【").append(newClerk.getNationcode()).append("】");
        }
        if (StringUtils.isNotBlank(newClerk.getPhoneNumber()) && !StringUtils.equals(newClerk.getPhoneNumber(), oldClerk.getPhoneNumber())) {
            sb.append("手机号").append("【").append(oldClerk.getPhoneNumber()).append("】")
                    .append("变更为【").append(newClerk.getPhoneNumber()).append("】");
        }
        if (StringUtils.isNotBlank(newClerk.getPositionName()) && !StringUtils.equals(newClerk.getPositionName(), oldClerk.getPositionName())) {
            sb.append("职位").append("【").append(oldClerk.getPositionName()).append("】")
                    .append("变更为【").append(newClerk.getPositionName()).append("】");
        }
        clerkLog.setContent(sb.toString());
        if (StringUtils.isNotBlank(clerkLog.getContent().trim())) {
            return tabClerkLogMapper.insertSelective(clerkLog);
        } else {
            return 0;
        }
    }

    @Override
    public Page<TabClerkLog> listClerkLog(ClerkLogDTO clerkLogDTO, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        return tabClerkLogMapper.listClerkLog(clerkLogDTO);
    }

}
