package com.gic.store.service.impl;

import com.gic.commons.util.GlobalInfo;
import com.gic.store.constant.StoreEnableOrDisAbleEnum;
import com.gic.store.dao.mapper.TabClerkMapper;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.entity.TabClerk;
import com.gic.store.service.ClerkService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @date 2019/7/16
 */
@Service
public class ClerkServiceImpl implements ClerkService {

    @Autowired
    private TabClerkMapper tabClerkMapper;

    @Override
    public boolean isRepeatByClerkCode(Integer storeInfoId, String clerkCode, Integer clerkId) {
        TabClerk tabClerk = new TabClerk();
        tabClerk.setClerkId(clerkId);
        tabClerk.setStoreInfoId(storeInfoId);
        tabClerk.setClerkCode(clerkCode);
        return tabClerkMapper.getBySelective(tabClerk) != null;
    }

    @Override
    public boolean isRepeatByPhoneNumber(Integer storeInfoId, String phoneNumber, Integer clerkId, String nationCode) {
        TabClerk tabClerk = new TabClerk();
        tabClerk.setClerkId(clerkId);
        tabClerk.setStoreInfoId(storeInfoId);
        tabClerk.setPhoneNumber(phoneNumber);
        tabClerk.setNationcode(nationCode);
        return tabClerkMapper.getBySelective(tabClerk) != null;
    }

    @Override
    public boolean isRepeatByClerkName(Integer storeInfoId, String clerkName, Integer clerkId) {
        if(StringUtils.isBlank(clerkName)){
            return false;
        }
        TabClerk tabClerk = new TabClerk();
        tabClerk.setClerkId(clerkId);
        tabClerk.setStoreInfoId(storeInfoId);
        tabClerk.setClerkName(clerkName);
        return tabClerkMapper.getBySelective(tabClerk) != null;
    }

    @Override
    public Integer save(ClerkDTO copy) {
        TabClerk tabClerk = new TabClerk();
        tabClerk.setClerkName(copy.getClerkName());
        tabClerk.setClerkCode(copy.getClerkCode());
        tabClerk.setEnterpriseId(copy.getEnterpriseId());
        tabClerk.setStoreInfoId(copy.getStoreInfoId());
        tabClerk.setPositionName(copy.getPositionName());
        tabClerk.setClerkGender(copy.getClerkGender());
        tabClerk.setNationcode(copy.getNationcode());
        tabClerk.setPhoneNumber(copy.getPhoneNumber());
        tabClerk.setCreateTime(new Date());
        tabClerk.setStatus(StoreEnableOrDisAbleEnum.ENABLE.getCode());
        tabClerkMapper.insertSelective(tabClerk);
        return tabClerk.getClerkId();
    }

    @Override
    public Integer update(ClerkDTO copy) {
        TabClerk tabClerk = new TabClerk();
        tabClerk.setClerkId(copy.getClerkId());
        tabClerk.setClerkName(copy.getClerkName());
        tabClerk.setPositionName(copy.getPositionName());
        tabClerk.setClerkGender(copy.getClerkGender());
        tabClerk.setNationcode(copy.getNationcode());
        tabClerk.setPhoneNumber(copy.getPhoneNumber());
//        tabClerk.setStatus(StoreEnableOrDisAbleEnum.ENABLE.getCode());
        return tabClerkMapper.updateByPrimaryKeySelective(tabClerk);
    }

    @Override
    public List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, List<Integer> storeInfoIds, String search) {
        return tabClerkMapper.listClerkByStoreInfoId(enterpriseId, storeInfoIds, null, search);
    }

    @Override
    public List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, String storeInfoIds, String search) {
        return tabClerkMapper.listClerkByStoreInfoId(enterpriseId, null, storeInfoIds, search);
    }

    @Override
    public Integer getTotalClerk(Integer enterpriseId, List<Integer> storeInfoIds, String search) {
        return tabClerkMapper.getTotalClerk(enterpriseId, storeInfoIds, search);
    }

    @Override
    public List<Integer> getStoreInfoIdsBySearch(Integer enterpriseId, String search) {
        return tabClerkMapper.getStoreInfoIdsBySearch(enterpriseId, search);
    }

    @Override
    public Integer updateClerkStatus(Integer enterpriseId, String clerkIds, Integer status) {
        List<Integer> clerkIdList;
        if (StringUtils.isNotBlank(clerkIds)) {
            String[] split = clerkIds.split(GlobalInfo.FLAG_COMMA);
            clerkIdList = Stream.of(split).map(Integer::parseInt).collect(Collectors.toList());
        } else {
            return 0;
        }
        return tabClerkMapper.updateClerkStatus(enterpriseId, clerkIdList, status);
    }

    @Override
    public List<TabClerk> listClerkByIds(Integer enterpriseId, String clerkIds) {
        if (StringUtils.isNotBlank(clerkIds)) {
            List<Integer> clerkIdList;
            String[] split = clerkIds.split(GlobalInfo.FLAG_COMMA);
            clerkIdList = Stream.of(split).map(Integer::parseInt).collect(Collectors.toList());
            return tabClerkMapper.listClerkByIds(enterpriseId, clerkIdList);
        } else {
            return Collections.emptyList();
        }
    }

    @Override
    public TabClerk getById(Integer clerkId) {
        return this.tabClerkMapper.selectByPrimaryKey(clerkId);
    }

    @Override
    public List<TabClerk> listClerkByStoreInfoId(Integer enterpriseId, String search) {
        return this.tabClerkMapper.listClerkByStoreInfoId(enterpriseId, null, null, search);
    }

    @Override
    public TabClerk getClerkByCode(Integer enterpriseId, String clerkCode, Integer storeInfoId) {
        return tabClerkMapper.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
    }

}
