package com.gic.store.service.outer.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.CollectionUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.store.constant.ClerkLogReasonEnum;
import com.gic.store.constant.ImportClerkBatchResultEnum;
import com.gic.store.constant.StoreLogTypeEnum;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.*;
import com.gic.store.dto.clerk.ProcessBatchClerkDTO;
import com.gic.store.entity.TabClerk;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.entity.TabStoreRegion;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/7/16
 */
@Service("clerkApiService")
public class ClerkApiServiceImpl implements ClerkApiService {

    private static final Logger logger = LogManager.getLogger(ClerkApiServiceImpl.class);

    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private ClerkLogService clerkLogService;
    @Autowired
    private StoreLogService storeLogService;
    @Autowired
    private StoreRegionService storeRegionService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    @Override
    public ServiceResponse saveOrUpdate(ClerkDTO clerkDTO) {
        if (validStoreIsOther(clerkDTO.getEnterpriseId(), clerkDTO.getStoreInfoId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "权限不足");
        }
        // 校验参数
        if (clerkService.isRepeatByClerkCode(clerkDTO.getStoreInfoId(), clerkDTO.getClerkCode(),
                clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购关联主键重复");
        }
        if (StringUtils.isNotBlank(clerkDTO.getPhoneNumber())) {
            String nationcode = clerkDTO.getNationcode();
            if (StringUtils.isBlank(nationcode)) {
                nationcode = Constants.NATION_CODE;
            }
            //默认加上+
            if (!nationcode.contains("+")) {
                nationcode = "+" + nationcode;
            }
            clerkDTO.setNationcode(nationcode);

            if (clerkService.isRepeatByPhoneNumber(clerkDTO.getStoreInfoId(), clerkDTO.getPhoneNumber(),
                    clerkDTO.getClerkId(), clerkDTO.getNationcode())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号重复");
            }
        }
        if (clerkService.isRepeatByClerkName(clerkDTO.getStoreInfoId(), clerkDTO.getClerkName(),
                clerkDTO.getClerkId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "导购名称重复");
        }
        if (clerkDTO.getClerkId() == null) {
            //            Integer saveClerkLogBySaveClerk(Integer reason, Integer enterpriseId, Integer clerkId, String remark, Integer operatorId, String operatorName);
            Integer id = clerkService.save(clerkDTO);
            clerkDTO.setClerkId(id);
            clerkLogService.saveClerkLogBySaveClerk(clerkDTO.getEnterpriseId(), id, clerkDTO.getReason(),
                    clerkDTO.getRemark(), clerkDTO.getOperatorId(), clerkDTO.getOperatorName());
            storeLogByAddClerk(clerkDTO);
        } else {
            TabClerk oldClerk = this.clerkService.getById(clerkDTO.getClerkId());
            clerkService.update(clerkDTO);
            clerkLogService.saveClerkLogByUpdateClerk(oldClerk, clerkDTO, clerkDTO.getEnterpriseId(),
                    clerkDTO.getClerkId(), clerkDTO.getReason(), clerkDTO.getRemark(), clerkDTO.getOperatorId(),
                    clerkDTO.getOperatorName());
        }
        return ServiceResponse.success(clerkDTO.getClerkId());
    }

    @Override
    public ServiceResponse<Integer> saveClerkForOrder(String clerkName, String clerkCode, Integer storeId,
            Integer enterpriseId, String orderNum) {
        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, storeId);
        if (storeResponse.getResult() != null) {
            StoreDTO store = storeResponse.getResult();
            ClerkDTO clerkDTO = new ClerkDTO();
            clerkDTO.setClerkName(clerkName);
            clerkDTO.setClerkCode(clerkCode);
            clerkDTO.setEnterpriseId(enterpriseId);
            clerkDTO.setStoreInfoId(store.getStoreInfoId());
            clerkDTO.setReason(ClerkLogReasonEnum.ORDER.getCode());
            clerkDTO.setRemark("订单号:" + orderNum);
            clerkDTO.setStoreName(store.getStoreName());
            ServiceResponse serviceResponse = this.saveOrUpdate(clerkDTO);
            return serviceResponse;
        } else {
            logger.warn("门店不存在 enterpriseId:{}, storeId:{}", enterpriseId, storeId);
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
    }

    @Override
    public ServiceResponse<ClerkDTO> getClerkByCode(Integer enterpriseId, String clerkCode, Integer storeId) {
        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, storeId);
        if (storeResponse.getResult() != null) {
            StoreDTO store = storeResponse.getResult();

            TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkCode, store.getStoreInfoId());
            if (clerk != null) {
                return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(ClerkDTO.class, clerk));
            } else {
                return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
            }
        } else {
            logger.warn("门店不存在 enterpriseId:{}, storeId:{}", enterpriseId, storeId);
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
    }

    private void storeLogByAddClerk(ClerkDTO clerkDTO) {
        StoreLogDTO storeLogDTO = new StoreLogDTO();
        storeLogDTO.setEnterpriseId(clerkDTO.getEnterpriseId());
        Integer storeId = this.storeService.getStoreIdByStoreInfoId(clerkDTO.getEnterpriseId(),
                clerkDTO.getStoreInfoId());
        storeLogDTO.setStoreId(storeId);
        storeLogDTO.setContent("门店关联导购【导购编码" + clerkDTO.getClerkCode() + "】");
        storeLogDTO.setReason(clerkDTO.getReason());
        storeLogDTO.setLogType(StoreLogTypeEnum.ADD_CLERK.getCode());
        storeLogDTO.setRemark(clerkDTO.getRemark());
        storeLogDTO.setOperatorId(clerkDTO.getOperatorId());
        storeLogDTO.setOperatorName(clerkDTO.getOperatorName());

        storeLogService.saveStoreLog(storeLogDTO);
    }

    private boolean validStoreIsOther(Integer enterpriseId, Integer storeInfoId) {
        Integer owner = storeService.getStoreOwnerByStoreInfoId(enterpriseId, storeInfoId);
        return owner == StoreOwnTypeEnum.OTHER.getCode();
    }

    @Override
    @SuppressWarnings("unchecked")
    public ServiceResponse<Page<ClerkStoreListDTO>> listClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO,
            Integer pageNum, Integer pageSize) {

        boolean hasStoreIds = StringUtils.isNotBlank(clerkSearchDTO.getStoreIds());
        //如果有门店数据条件，则不判断门店分组数据，因为分店分组数据也是为了获取门店
        if (!hasStoreIds) {
            if (clerkSearchDTO.getStoreGroupId() == null) {
                return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
            }
        }
        // 分组的id
        List<TabStoreGroup> groupList = storeGroupService.listStoreGroupAndChildren(clerkSearchDTO.getStoreGroupId());
        List<Integer> groupIds = groupList.stream().map(TabStoreGroup::getStoreGroupId).collect(Collectors.toList());
        //         查询条件对应的storeId
        String storeIdByClerkSearch;
        if (StringUtils.isNotBlank(clerkSearchDTO.getSearch())) {
            List<Integer> storeInfoIdsBySearch = clerkService.getStoreInfoIdsBySearch(enterpriseId, null);
            if (CollectionUtils.isEmpty(groupIds) || CollectionUtils.isEmpty(storeInfoIdsBySearch)) {
                storeIdByClerkSearch = "";
            } else {
                storeIdByClerkSearch = getStoreIdByStoreInfoIds(storeInfoIdsBySearch, null);
            }
        } else {
            storeIdByClerkSearch = "";
        }
        clerkSearchDTO.setEnterpriseId(enterpriseId);
        if (!hasStoreIds) {
            clerkSearchDTO.setStoreIds(storeIdByClerkSearch);
        }
        ServiceResponse<Page<StoreDTO>> serviceResponse = listStoreIds(clerkSearchDTO, pageNum, pageSize);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        if (page == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "ES门店数据为空");
        }
        List<StoreDTO> storeList = page.getResult();
        List<Integer> storeInfoIds = new ArrayList<>(storeList.size());
        List<ClerkStoreListDTO> storeClerkDTOList = new ArrayList<>(storeList.size());
        for (StoreDTO store : storeList) {
            ClerkStoreListDTO clerkStoreListDTO = new ClerkStoreListDTO();
            clerkStoreListDTO.setStoreId(store.getStoreId());
            clerkStoreListDTO.setStoreInfoId(store.getStoreInfoId());
            clerkStoreListDTO.setStoreName(store.getStoreName());
            clerkStoreListDTO.setStoreCode(store.getStoreCode());
            clerkStoreListDTO.setOwnType(store.getOwnType());
            storeClerkDTOList.add(clerkStoreListDTO);
            storeInfoIds.add(store.getStoreInfoId());
        }
        List<TabClerk> clerkList = clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds,
                clerkSearchDTO.getSearch());
        Map<Integer, List<TabClerk>> storeIdMap = CollectionUtil.group(clerkList, "storeInfoId");
        for (ClerkStoreListDTO clerkStoreListDTO : storeClerkDTOList) {
            List<TabClerk> tabClerks = storeIdMap.get(clerkStoreListDTO.getStoreInfoId());
            if (CollectionUtils.isNotEmpty(tabClerks)) {
                clerkStoreListDTO.setClerkList(EntityUtil.changeEntityListByOrika(ClerkDTO.class, tabClerks));
            } else {
                clerkStoreListDTO.setClerkList(Collections.emptyList());
            }
        }
        Page resultPage = new Page();
        resultPage.setPageSize(page.getPageSize());
        resultPage.setCurrentPage(page.getCurrentPage());
        resultPage.setTotalCount(page.getTotalCount());
        resultPage.setTotalPage(page.getPages());
        resultPage.setResult(storeClerkDTOList);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Integer> getTotalClerk(Integer enterpriseId, ClerkSearchDTO clerkSearchDTO) {
        if (clerkSearchDTO.getStoreGroupId() == null) {
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        // 分组的id
        List<TabStoreGroup> groupList = storeGroupService.listStoreGroupAndChildren(clerkSearchDTO.getStoreGroupId());
        List<Integer> groupIds = groupList.stream().map(TabStoreGroup::getStoreGroupId).collect(Collectors.toList());
        //         查询条件对应的storeId
        String storeIdByClerkSearch;
        if (StringUtils.isNotBlank(clerkSearchDTO.getSearch())) {
            List<Integer> storeInfoIdsBySearch = clerkService.getStoreInfoIdsBySearch(enterpriseId,
                    clerkSearchDTO.getSearch());
            if (CollectionUtils.isEmpty(groupIds) || CollectionUtils.isEmpty(storeInfoIdsBySearch)) {
                storeIdByClerkSearch = "";
            } else {
                storeIdByClerkSearch = getStoreIdByStoreInfoIds(storeInfoIdsBySearch, clerkSearchDTO.getSearch());
            }
        } else {
            storeIdByClerkSearch = "";
        }
        clerkSearchDTO.setEnterpriseId(enterpriseId);
        clerkSearchDTO.setStoreIds(storeIdByClerkSearch);
        ServiceResponse<Page<StoreDTO>> serviceResponse = listStoreIds(clerkSearchDTO, 1, Integer.MAX_VALUE);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        Page<StoreDTO> page = serviceResponse.getResult();
        if (page == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "ES门店数据为空");
        }
        List<Integer> storeInfoIds = page.getResult().stream().map(StoreDTO::getStoreInfoId)
                .collect(Collectors.toList());
        Integer totalClerk = clerkService.getTotalClerk(enterpriseId, storeInfoIds, clerkSearchDTO.getSearch());

        return ServiceResponse.success(totalClerk);
    }

    private String getStoreIdByStoreInfoIds(List<Integer> storeInfoIdsBySearch, String search) {
        StoreSearchDBDTO storeSearchDBDTO = new StoreSearchDBDTO();
        storeSearchDBDTO.setStoreInfoIdList(storeInfoIdsBySearch);
        storeSearchDBDTO.setSearch(search);
        List<StoreDTO> dtos = storeService.listStore(storeSearchDBDTO, 1, Integer.MAX_VALUE);
        return Optional.ofNullable(dtos).orElse(Collections.emptyList()).stream().map(e -> e.getStoreId().toString())
                .reduce((x, y) -> x + " " + y).orElse("");
    }

    @SuppressWarnings("unchecked")
    private ServiceResponse<Page<StoreDTO>> listStoreIds(ClerkSearchDTO clerkSearchDTO, Integer pageNum,
            Integer pageSize) {
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(clerkSearchDTO.getEnterpriseId());
        storeSearchDTO.setStoreGroupId(clerkSearchDTO.getStoreGroupId());
        storeSearchDTO.setSearchJson(clerkSearchDTO.getSelectJson());
        storeSearchDTO.setStoreIds(clerkSearchDTO.getStoreIds());
//        storeSearchDTO.setSearch(clerkSearchDTO.getSearch());

        return storeApiService.listStore(storeSearchDTO, pageNum, pageSize,
                "storeId,storeName,storeCode,storeInfoId,ownType");
    }

    @Override
    public ServiceResponse updateClerkStatus(Integer enterpriseId, String clerkIds, Integer status) {
        return ServiceResponse.success(clerkService.updateClerkStatus(enterpriseId, clerkIds, status));
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByIds(Integer enterpriseId, String clerkIds) {
        List<TabClerk> clerkList = clerkService.listClerkByIds(enterpriseId, clerkIds);
        List<ClerkDTO> clerkDTOList = EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList);
        return EnterpriseServiceResponse.success(clerkDTOList);
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByStoreId(Integer enterpriseId, Integer storeId, String search) {
        ServiceResponse<StoreDTO> response = storeApiService.getStoreById(enterpriseId, storeId);
        if (response.isSuccess() && response.getResult() != null) {
            List<Integer> storeInfoIds = new ArrayList<>();
            storeInfoIds.add(response.getResult().getStoreInfoId());
            List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds, search);
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<ClerkDTO> getClerkById(Integer enterpriseId, Integer clerkId) {
        TabClerk tabClerk = this.clerkService.getById(clerkId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(ClerkDTO.class, tabClerk));
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerk(Integer enterpriseId, String search) {
        List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, search);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList));
    }

    @Override
    public ServiceResponse<String> importClerkForOperation(ProcessBatchClerkDTO clerkInfo) {
        if (clerkInfo == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "对象为空");
        }
        logger.info("批量处理导购的参数：{}", JSONObject.toJSONString(clerkInfo));
        //param valid
        Integer regionId = clerkInfo.getRegionId();
        Integer enterpriseId = clerkInfo.getEnterpriseId();
        //商户ID校验
        ServiceResponse enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResult.isSuccess()) {
            return responseErrorElement(ImportClerkBatchResultEnum.E1);
        }
        if (regionId == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.A1);
        }
        TabStoreRegion tabStoreRegion = storeRegionService.getById(enterpriseId, regionId);
        if (tabStoreRegion == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.A2);
        }
        String storeCode = clerkInfo.getStoreCode();
        if (StringUtils.isBlank(storeCode)) {
            return responseErrorElement(ImportClerkBatchResultEnum.A3);
        }
        StoreDTO storeDTO = storeService.getStoreByRegionIdAndStoreCode(enterpriseId, tabStoreRegion.getRegionId(),
                clerkInfo.getStoreCode());
        if (storeDTO == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.A4);
        }
        Integer storeInfoId = storeDTO.getStoreInfoId();
        String clerkCode = clerkInfo.getClerkCode();
        if (StringUtils.isBlank(clerkCode)) {
            return responseErrorElement(ImportClerkBatchResultEnum.C1);
        }
        TabClerk clerk = this.clerkService.getClerkByCode(enterpriseId, clerkCode, storeInfoId);
        boolean isAdd = true;
        //如果代购code存在，则进行更新操作
        if (clerk != null) {
            //return responseErrorElement(ImportClerkBatchResultEnum.C2);
            isAdd = false;

        }
        if (StringUtils.isBlank(clerkInfo.getClerkName())) {
            return responseErrorElement(ImportClerkBatchResultEnum.C3);
        }
        String nationcode = clerkInfo.getNationcode();
        if (StringUtils.isBlank(nationcode)) {
            nationcode = Constants.NATION_CODE;
        }
        Integer gender = clerkInfo.getClerkGender();
        if (gender != null && gender.intValue() != 1 && gender.intValue() != 2) {
            return responseErrorElement(ImportClerkBatchResultEnum.D2);
        }
        String phone = clerkInfo.getPhoneNumber();
        if (StringUtils.isBlank(phone)) {
            return responseErrorElement(ImportClerkBatchResultEnum.D4);
        }
        if (clerkService.isRepeatByPhoneNumber(storeInfoId, phone, null, nationcode)) {
            return responseErrorElement(ImportClerkBatchResultEnum.D3);
        }
        ClerkDTO clerkDTO = EntityUtil.changeEntityNew(ClerkDTO.class, clerkInfo);
        clerkDTO.setStoreInfoId(storeInfoId);
        clerkDTO.setStoreName(storeDTO.getStoreName());
        Integer id;
        if (isAdd) {
            id = clerkService.save(clerkDTO);
            clerkLogService.saveClerkLogBySaveClerk(enterpriseId, id, clerkDTO.getReason(),
                    clerkDTO.getRemark(), clerkDTO.getOperatorId(), clerkDTO.getOperatorName());
            storeLogByAddClerk(clerkDTO);
        } else {
            id = clerk.getClerkId();
            clerkDTO.setClerkId(id);
            clerkService.update(clerkDTO);
            clerkLogService.saveClerkLogByUpdateClerk(clerk, clerkDTO, clerkDTO.getEnterpriseId(),
                    clerkDTO.getClerkId(), clerkDTO.getReason(), clerkDTO.getRemark(), clerkDTO.getOperatorId(),
                    clerkDTO.getOperatorName());
        }
        //性别为空，但是标记导入
        if (gender == null) {
            return responseErrorElement(ImportClerkBatchResultEnum.D1);
        }
        return ServiceResponse.success(id.toString());
    }

    @Override
    public ServiceResponse<Integer> addClerkToOtherStore(Integer clerkId, Integer storeId, Integer enterpriseId) {
        logger.info("clerkId:{}, storeId:{}, enterpriseId:{}", clerkId, storeId, enterpriseId);
        TabClerk clerk = this.clerkService.getById(clerkId);
        if (clerk == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "导购不存在");
        }
        StoreDTO store = this.storeApiService.getStoreById(enterpriseId, storeId).getResult();
        if (store == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "门店不存在");
        }
        if (clerk.getStoreInfoId().equals(store.getStoreInfoId())) {
            logger.info("导购[clerkId:{}]已存在门店[storeId:{}, storeInfoId:{}]下", clerkId, store.getStoreId(), store.getStoreInfoId());
            return EnterpriseServiceResponse.success(clerkId);
        }
        ClerkDTO clerkDTO = EntityUtil.changeEntityByJSON(ClerkDTO.class, clerk);
        clerkDTO.setClerkId(null);
        clerkDTO.setStoreInfoId(store.getStoreInfoId());
        ServiceResponse<Integer> clerkResp = this.saveOrUpdate(clerkDTO);
        if (clerkResp.isSuccess()) {
            logger.info("导购[clerkId:{}]不存在门店[storeId:{}, storeInfoId:{}]下, 新增clerkId：{}", clerkId, store.getStoreId(), store.getStoreInfoId(), clerkResp.getResult());
            return EnterpriseServiceResponse.success(clerkResp.getResult());
        } else {
            logger.info("保存导购失败：{}", clerkResp.getMessage());
            return EnterpriseServiceResponse.failure(clerkResp.getCode(), clerkResp.getMessage());
        }
    }

    @Override
    public ServiceResponse<List<ClerkDTO>> listClerkByStoreBrand(Integer enterpriseId, Integer storeBrandId, String clerkName) {
        StoreSearchDBDTO storeSearchDBDTO = new StoreSearchDBDTO();
        storeSearchDBDTO.setEnterpriseId(enterpriseId);
        storeSearchDBDTO.setStoreBrandId(storeBrandId);
        List<StoreDTO> list = this.storeService.listStore(storeSearchDBDTO, 1, 10000);
        if (CollectionUtils.isEmpty(list)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        // list 如果太大， 让 mybatis 解析 会打满内存
        String storeInfoIds = list.stream().map(e -> e.getStoreInfoId().toString()).reduce((x, y) -> x + "," + y).get();
        storeInfoIds = "(" + storeInfoIds + ")";
        List<TabClerk> clerkList = this.clerkService.listClerkByStoreInfoId(enterpriseId, storeInfoIds, clerkName);
        List<ClerkDTO> dtoList = EntityUtil.changeEntityListByJSON(ClerkDTO.class, clerkList);
        return EnterpriseServiceResponse.success(dtoList);
    }

    private static ServiceResponse<String> responseErrorElement(ImportClerkBatchResultEnum error) {
        logger.warn("批处理导购错误返回结果:{}", error.toString());
        return ServiceResponse.failure(error.getCode(), error.toString());
    }

}