package com.gic.store.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.dto.CityDTO;
import com.gic.store.dto.CountyDTO;
import com.gic.store.dto.ProvinceDTO;
import com.gic.store.entity.TabCity;
import com.gic.store.entity.TabCounty;
import com.gic.store.entity.TabProvince;
import com.gic.store.service.ProvincesApiService;
import com.gic.store.service.ProvincesService;
import com.gic.store.service.StoreService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service("provincesApiService")
public class ProvincesApiServiceImpl implements ProvincesApiService {
    @Autowired
    private ProvincesService provincesService;
    @Autowired
    private StoreService storeService;

    @Override
    public ServiceResponse<List<ProvinceDTO>> selectAllProvince() {
        String key = "enterprise:province";
        List<ProvinceDTO> list = (List<ProvinceDTO>)RedisUtil.getCache(key);
        if(list == null){
            list = EntityUtil.changeEntityListByJSON(ProvinceDTO.class, this.provincesService.selectAllProvince());
            RedisUtil.setCache(key, list, 6*30l, TimeUnit.DAYS);
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<CityDTO>> selectAllCity() {
        String key = "enterprise:city";
        List<CityDTO> list = (List<CityDTO>)RedisUtil.getCache(key);
        if(list == null){
            list = EntityUtil.changeEntityListByJSON(CityDTO.class, this.provincesService.selectAllCity());
            RedisUtil.setCache(key, list, 6*30l, TimeUnit.DAYS);
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<CityDTO> getCityByCityId(String cityId) {
        ServiceResponse<List<CityDTO>> response = this.selectAllCity();
        if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
            List<CityDTO> list = response.getResult();
            for(CityDTO dto : list){
                if(dto.getCityId().equals(cityId)){
                    return ServiceResponse.success(dto);
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<CityDTO>> selectCityByProvinceId(String provinceId) {
        List<CityDTO> list = new ArrayList<>();
        if(StringUtils.isBlank(provinceId)){
            return ServiceResponse.success(list);
        }
        ServiceResponse<List<CityDTO>> response = this.selectAllCity();
        if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
            for(CityDTO cityDTO : response.getResult()){
                if(cityDTO.getProvinceId().equals(provinceId)){
                    list.add(cityDTO);
                }
            }
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<CountyDTO>> selectAllCounty() {
        String key = "enterprise:county";
        List<CountyDTO> list = (List<CountyDTO>)RedisUtil.getCache(key);
        if(list == null){
            list = EntityUtil.changeEntityListByJSON(CountyDTO.class, this.provincesService.selectAllCounty());
            RedisUtil.setCache(key, list, 6*30l, TimeUnit.DAYS);
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<CountyDTO> getCountyByCountyId(String countyId) {
        ServiceResponse<List<CountyDTO>> response = this.selectAllCounty();
        if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
            for(CountyDTO dto : response.getResult()){
                if(dto.getCountyId().equals(countyId)){
                    return ServiceResponse.success(dto);
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<CountyDTO>> selectCountyByCityId(String cityId) {
        List<CountyDTO> list = new ArrayList<>();
        if(StringUtils.isBlank(cityId)){
            return ServiceResponse.success(list);
        }
        ServiceResponse<List<CountyDTO>> response = this.selectAllCounty();
        if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
            for(CountyDTO countyDTO : response.getResult()){
                if(countyDTO.getCityId().equals(cityId)){
                    list.add(countyDTO);
                }
            }
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<String>> selectCountyIdByCityId(String cityId) {
        ServiceResponse<List<CountyDTO>> response = this.selectCountyByCityId(cityId);
        List<String> list = new ArrayList<>();
        if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
            for(CountyDTO dto : response.getResult()){
                list.add(dto.getCountyId());
            }
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<ProvinceDTO>> queryProvinces(String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ProvinceDTO.class, this.provincesService.queryProvinces(search)));
    }

    @Override
    public ServiceResponse<List<CityDTO>> queryCity(String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(CityDTO.class, this.provincesService.queryCity(search)));
    }

    @Override
    public ServiceResponse<List<CountyDTO>> queryCounty(String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(CountyDTO.class, this.provincesService.queryCounty(search)));
    }

    @Override
    public ServiceResponse<ProvinceDTO> getProvinceById(String provinceId) {
        ServiceResponse<List<ProvinceDTO>> serviceResponse = this.selectAllProvince();
        if(serviceResponse.isSuccess()){
            List<ProvinceDTO> list = serviceResponse.getResult();
            if(CollectionUtils.isNotEmpty(list)){
                for(ProvinceDTO provinceDTO : list){
                    if(provinceDTO.getProvinceId().equals(provinceId)){
                        return ServiceResponse.success(provinceDTO);
                    }
                }
            }
        }
        return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "省份不存在");
    }

    @Override
    public ServiceResponse<String> getProvinceNameByAreaId(String areaId) {
        List<TabCounty> tabCounties = this.provincesService.selectAllCounty();
        List<TabCity> tabCities = this.provincesService.selectAllCity();
        List<TabProvince> tabProvinces = this.provincesService.selectAllProvince();
        Map<String, TabCounty> countyMap = tabCounties.stream().collect(Collectors.toMap(TabCounty::getCountyId, a -> a, (k1, k2) -> k1));
        Map<String, TabCity> tabCityMap = tabCities.stream().collect(Collectors.toMap(TabCity::getCityId, a -> a, (k1, k2) -> k1));
        Map<String, TabProvince> provinceMap = tabProvinces.stream().collect(Collectors.toMap(TabProvince::getProvinceId, a -> a, (k1, k2) -> k1));
        StringBuilder stringBuilder = new StringBuilder();
        if(countyMap.containsKey(areaId)){
            TabCounty tabCounty = countyMap.get(areaId);
            stringBuilder.append(provinceMap.get(tabCounty.getProvinceId()).getProvinceName()).append("/");
            stringBuilder.append(tabCityMap.get(tabCounty.getCityId()).getCityName()).append("/");
            stringBuilder.append(tabCounty.getCountyName());
        }
        return ServiceResponse.success(stringBuilder.toString());
    }

    @Override
    public ServiceResponse<List<CityDTO>> listHotCity(Integer enterpriseId, Integer brandId) {
        List<String> list = this.storeService.listAllCityId(enterpriseId, brandId);
        ServiceResponse<List<CityDTO>> response = this.selectAllCity();
        List<CityDTO> hotCityList = new ArrayList<>();
        if(response.isSuccess()){
            for(CityDTO dto : response.getResult()){
                if(list.contains(dto.getCityId())){
                    hotCityList.add(dto);
                }
            }
        }
        return ServiceResponse.success(hotCityList);
    }

    @Override
    public ServiceResponse<List<CityDTO>> listEnterpriseCity(Integer enterpriseId) {
        List<String> list = this.storeService.listAllCityId(enterpriseId, null);
        ServiceResponse<List<CityDTO>> response = this.selectAllCity();
        List<CityDTO> cityList = new ArrayList<>();
        if(response.isSuccess()){
            for(CityDTO dto : response.getResult()){
                if(list.contains(dto.getCityId())){
                    cityList.add(dto);
                }
            }
        }
        return ServiceResponse.success(cityList);
    }
}
