package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.Constants;
import com.gic.store.constant.GenderEnum;
import com.gic.store.constant.StoreImportEnum;
import com.gic.store.constant.StoreOwnTypeEnum;
import com.gic.store.dto.ClerkTmpDTO;
import com.gic.store.dto.GicDictDTO;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.ClerkImportApiService;
import com.gic.store.service.DictApiService;
import com.gic.store.service.StoreApiService;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.utils.ExcelUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zhiwj
 * @date 2019/7/17
 */
@RestController
public class ClerkImportController {

    private static final Logger logger = LogManager.getLogger(ClerkImportController.class);

    @Autowired
    private ClerkImportApiService clerkImportApiService;

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private DictApiService dictApiService;

    /**
     * 下载模板
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping("/clerk-import-template-download")
    public Object download(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String fileName = " 导购资料导入模板.xlsx";

        OutputStream fo = null;
        InputStream in = null;
        try {
            response.setContentType("text/html;charset=gbk");
            response.setHeader("Content-Disposition",
                    "attachment; filename=" + new String(fileName.getBytes("gbk"), "ISO-8859-1"));

            fo = response.getOutputStream();
//            String rootPath = request.getSession().getServletContext().getRealPath("/");
//            String filePath = "excel/store_tag_batch_import.xlsx";
//            in = new FileInputStream(new File(rootPath + filePath));
            in = this.getClass().getClassLoader().getResourceAsStream("excel/clerk_batch_import.xlsx");
            byte[] b = new byte[1024];
            int len = 0;
            while ((len = in.read(b)) != -1) {
                fo.write(b, 0, len);
            }
            fo.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (fo != null) {
                fo.close();
            }
            if (in != null) {
                in.close();
            }
        }
        return RestResponse.success();
    }

    /**
     * 批量导入
     * @param request
     * @param file
     * @param pageQO
     * @return
     * @throws IOException
     */
    @RequestMapping("/clerk-tmp-file-upload")
    public RestResponse fileUpload(HttpServletRequest request, MultipartFile file, PageQO pageQO) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (file == null || file.isEmpty()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "文件不存在");
        }
        // 判断是否有正在导入的任务
        ServiceResponse<Boolean> importingStoreResponse = clerkImportApiService.hasImportingClerk(enterpriseId);
        if (importingStoreResponse.getResult()) {
            return RestResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "当前企业下有正在导入导购的任务，请稍后重试！");
        }
        if (file.getSize() > 0) {
            String fileName = file.getOriginalFilename();
            String uuId = request.getParameter("uuId");
            if (fileName.endsWith("xlsx") || fileName.endsWith("xls")) {
                try {
                    Workbook workbook = ExcelUtils.getWorkbookFromUpload(file.getInputStream(), fileName);
                    List<Map<Integer, String>> data = ExcelUtils.readExcel(6, fileName.substring(fileName.lastIndexOf(".") + 1), workbook, null);
                    if (CollectionUtils.isNotEmpty(data) && data.get(0).size() != 9) {
                        return RestResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "总列数与模板不一致");
                    }
                    clerkImportApiService.delClerkTmp(enterpriseId);
                    for (Map<Integer, String> row : data) {
                        ClerkTmpDTO bean = this.validateData(row, enterpriseId);
                        try {
                            bean.setSignKey(uuId);
                            // 保存数据到临时表
                            bean.setOperatorId(UserDetailUtils.getUserDetail().getUserId());
                            bean.setOperatorName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
                            clerkImportApiService.save(bean);
                        } catch (Exception e) {
                            logger.warn(e);
                        }
                    }

                    if (clerkImportApiService.hasImportingClerk(enterpriseId).getResult()) {
                        // 开队列
                        String r = clerkImportApiService.importDataToClerk(enterpriseId, "123", uuId).getResult();
                        if (StringUtils.isNotBlank(r)) {
                            logger.warn("队列错误, {}", r);
                            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "网络错误");
                        }
                    } else {
                        Page page = clerkImportApiService.listClerkTmp(enterpriseId, false, pageQO.getCurrentPage(), pageQO.getPageSize()).getResult();
                        Map<Object, Object> result = new HashMap<>(8);
                        result.put("errorData", page);
                        int errorRows = page.getTotalCount();
                        result.put("total", errorRows);
                        result.put("success", 0);
                        result.put("error", errorRows);
                        return RestResponse.success(result);
                    }
                } catch (Exception e) {
                    logger.warn("excel验证不通过", e);
                    return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "excel验证不通过");
                }
            } else {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "选择xlsx格式文件上传");
            }
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "选择xlsx格式文件上传");
        }
        LogUtils.createLog("导入导购", "导购");
        return RestResponse.success();
    }

    /**
     * 查看进度
     * @param request
     * @param page
     * @return
     */
    @RequestMapping("/get_clerk_batch_process")
    public Object getStoreBatchProcess(HttpServletRequest request, PageQO page) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String uuId = request.getParameter("uuId");
        Map<String, Object> data = clerkImportApiService.getClerkBatchProcess(enterpriseId, uuId, page.getCurrentPage(), page.getPageSize()).getResult();
        return RestResponse.success(data);
    }

    /**
     * 错误日志查看
     * @param page
     * @return
     */
    @RequestMapping("/clerk-import-load-error-data")
    public Object loadErrorData(PageQO page) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Map<String, Object> result = new HashMap<String, Object>();
        List<ClerkTmpDTO> successList = clerkImportApiService.listClerkTmp(enterpriseId, true).getResult();
        ServiceResponse<Page> serviceResponse = clerkImportApiService.pageError(enterpriseId, page.getCurrentPage(), page.getPageSize());
        List<ClerkTmpDTO> totalList = clerkImportApiService.listAllClerkTmp(enterpriseId).getResult();
        Page pageResponse = serviceResponse.getResult();
        result.put("errorData", pageResponse);
        result.put("total", totalList == null ? 0 : totalList.size());
        result.put("success", successList == null ? 0 : successList.size());
        result.put("error", pageResponse.getResult().size());
        return RestResponse.success(result);
    }

    /**
     * 导出错误日志
     * @param request
     * @param response
     * @return
     * @throws IOException
     */
    @RequestMapping("/export-clerk-error-data")
    public RestResponse exportErrorData(HttpServletRequest request, HttpServletResponse response) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String fileName = "导购导入错误记录.xlsx";
        List<ClerkTmpDTO> listErrorData = clerkImportApiService.listClerkTmp(enterpriseId, false).getResult();
        if (!listErrorData.isEmpty()) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
            fileName = sdf.format(listErrorData.get(0).getUpdateTime()) + fileName;
            List<List<String>> errorList = clerkImportApiService.listError(enterpriseId).getResult();
            List<String> titleList = new ArrayList<>();
            Collections.addAll(titleList, "序号", "导购名", "导购关联主键", "门店名", "门店关联主键", "职位", "性别", "区号", "手机号", "错误信息");
            List<String> descriptionList = new ArrayList<>();
            Collections.addAll(descriptionList, "", "必填，不能超过20个字", "必填，不能超过20个字，为导购的唯一识别码", "必填，不能超过20个字", "必填,不能超过20个字", "非必填，不能超过30个字", "\"男\"或\"女\"", "区号，如不填默认为 086", "必填，不能超过20个字", "");

            Workbook workbook = ExcelUtils.getWorkbook("导购导入错误记录", titleList, descriptionList, errorList, null);
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);

            OutputStream out = response.getOutputStream();
            workbook.write(out);
            out.close();
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "没有错误日志");
        }
    }

    private ClerkTmpDTO validateData(Map<Integer, String> row, Integer enterpriseId) {
        boolean validate = true;
        ClerkTmpDTO bean = new ClerkTmpDTO();
        bean.setEnterpriseId(enterpriseId);
        bean.setErrorMessage(StoreImportEnum.WAIT.getCode());

        // 验证导购名
        String clerkName = row.get(1);

        if (StringUtils.isBlank(clerkName)) {
            bean.setErrorMessage("导购名称不能为空");
            validate = false;
        }
        clerkName = clerkName.trim();
        bean.setClerkName(clerkName);
        if (validate) {
            if (this.lengthValidate(clerkName, 20)) {
                bean.setErrorMessage("导购名称长度超过20个字");
                validate = false;
            }
        }
        // 验证导购code
        String clerkCode = row.get(2);
        if (validate) {
            if (StringUtils.isBlank(clerkCode)) {
                bean.setErrorMessage("导购关联主键不能为空");
                validate = false;
            }
        }
        clerkCode = clerkCode.trim();
        bean.setClerkCode(clerkCode);
        if (validate) {
            if (this.lengthValidate(clerkCode, 20)) {
                bean.setErrorMessage("导购关联主键超过20个字");
                validate = false;
            }
        }
        // 验证门店name
        String storeName = row.get(3);
        if (validate) {
            if (StringUtils.isBlank(storeName)) {
                bean.setErrorMessage("门店名不能为空");
                validate = false;
            }
        }
        storeName = storeName.trim();
        bean.setStoreName(storeName);
        if (validate) {
            if (this.lengthValidate(storeName, 20)) {
                bean.setErrorMessage("门店名超过20个字");
                validate = false;
            }
        }
        // 验证门店code
        String storeCode = row.get(4);
        if (validate) {
            if (StringUtils.isBlank(storeCode)) {
                bean.setErrorMessage("门店关联主键不能为空");
                validate = false;
            }
        }
        storeCode = storeCode.trim();
        bean.setStoreCode(storeCode);
        if (validate) {
            if (this.lengthValidate(storeCode, 20)) {
                bean.setErrorMessage("门店关联主键超过20个字");
                validate = false;
            }
        }
        // 验证门店是否存在
        StoreDTO storeDTO = storeApiService.getByStoreNameAndStoreCode(enterpriseId, storeName, storeCode).getResult();
        if (validate) {
            if (storeDTO == null) {
                bean.setErrorMessage("门店不存在");
                validate = false;
            }
        }
        if (validate) {
            if (StoreOwnTypeEnum.OWNER.getCode() != storeDTO.getOwnType()) {
                bean.setErrorMessage("不能在被授权门店下导入导购");
                validate = false;
            }
        }
        // 验证岗位
        String positionName = row.get(5);
        if (validate) {
            if (StringUtils.isNotBlank(positionName) && this.lengthValidate(storeCode, 20)) {
                bean.setErrorMessage("岗位超过20个字");
                validate = false;
            }
        }
        positionName = positionName.trim();
        bean.setPositionName(positionName);
        // 验证性别
        String gender = row.get(6);
        bean.setClerkGender(gender);
        if (validate) {
            if (StringUtils.isBlank(gender)) {
                bean.setErrorMessage("性别不能为空");
                validate = false;
            }
        }
        if (validate) {
            GenderEnum genderEnum = GenderEnum.parseMessage(gender);
            if (genderEnum == null) {
                bean.setErrorMessage("不存在该性别: " + gender);
            }
        }
        // 验证区号
        String nationcode = row.get(7);
        if (StringUtils.isBlank(nationcode)) {
            nationcode = "+86";
        }
        nationcode = nationcode.trim();
        if ('+' != nationcode.charAt(0)) {
            nationcode = "+" + nationcode;
        }
        bean.setNationcode(nationcode);
        if (validate) {
            if (this.lengthValidate(nationcode, 10)) {
                bean.setErrorMessage("区号超过10个字");
                validate = false;
            }
        }
        GicDictDTO dict = this.dictApiService.getDictByCode(Constants.NATION_CODE_DICT, nationcode).getResult();
        if (dict == null) {
            bean.setErrorMessage("区号错误");
            validate = false;
        }

        // 验证手机号
        String phoneNumber = row.get(8);
        bean.setPhoneNumber(phoneNumber);
        if (validate) {
            if (StringUtils.isBlank(phoneNumber)) {
                bean.setErrorMessage("手机号不能为空");
            }
        }
        return bean;
    }

    private boolean lengthValidate(String value, int len) {
        String _value = value.replaceAll("[^\\x00-\\xff]", "**");
        return _value.length() / 2.0 > (double) len;
    }
}
