package com.gic.store.web.controller;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.AuditLogApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.dto.StoreDictDTO;
import com.gic.store.service.StoreDictApiService;
import com.gic.store.web.vo.StoreDictVO;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Controller
public class StoreDictController {
    private Logger logger = LogManager.getLogger(StoreDictController.class);
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;
    @Autowired
    private AuditLogApiService auditLogApiService;

    @RequestMapping("list-all-businesscategory")
    @ResponseBody
    public Object listAllBusinessCategory(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listAllBusinessCategory(enterpriseId);
        if(serviceResponse.isSuccess()){
            return RestResponse.success(EntityUtil.changeEntityListByJSON(StoreDictVO.class, serviceResponse.getResult()));
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("list-all-storetype")
    @ResponseBody
    public Object listAllStoreType(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listAllStoreType(enterpriseId);
        if(serviceResponse.isSuccess()){
            return RestResponse.success(EntityUtil.changeEntityListByJSON(StoreDictVO.class, serviceResponse.getResult()));
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("list-all-storestatus")
    @ResponseBody
    public RestResponse listAllStoreStatus(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listAllStoreStatus(enterpriseId);
        if(serviceResponse.isSuccess()){
            return RestResponse.success(EntityUtil.changeEntityListByJSON(StoreDictVO.class, serviceResponse.getResult()));
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("list-storetype")
    @ResponseBody
    public RestResponse listStoreType(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreType(enterpriseId);
        if(serviceResponse.isSuccess()){
            return RestResponse.success(EntityUtil.changeEntityListByJSON(StoreDictVO.class, serviceResponse.getResult()));
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("list-storestatus")
    @ResponseBody
    public RestResponse listStoreStatus(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreStatus(enterpriseId);
        if(serviceResponse.isSuccess()){
            return RestResponse.success(EntityUtil.changeEntityListByJSON(StoreDictVO.class, serviceResponse.getResult()));
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("save-storetype")
    @ResponseBody
    public RestResponse saveStoreType(String storeTypes){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if(StringUtils.isBlank(storeTypes)){
            return RestResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        ServiceResponse serviceResponse = this.storeDictApiService.saveStoreType(enterpriseId, storeTypes.split(","));
        if(serviceResponse.isSuccess()){
            LogUtils.createLog("配置商户门店类型", "门店类型");
            return RestResponse.success();
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("save-storestatus")
    @ResponseBody
    public RestResponse saveStoreStatus(String storeStatus){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if(StringUtils.isBlank(storeStatus)){
            return RestResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        ServiceResponse serviceResponse = this.storeDictApiService.saveStoreStatus(enterpriseId, storeStatus.split(","));
        if(serviceResponse.isSuccess()){
            LogUtils.createLog("配置商户门店状态", "门店状态");
            return RestResponse.success();
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("save-status-type")
    @ResponseBody
    public RestResponse saveStoreStatusAndStoreType(String storeStatus, String storeType){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse serviceResponse = this.storeDictApiService.saveStoreStatusAndStoreType(enterpriseId, storeStatus.split(","), storeType.split(","));
        if(serviceResponse.isSuccess()){
            LogUtils.createLog("配置商户门店类型和门店状态", "门店状态和门店类型");
            this.enterpriseInitApiService.setStoreStatusAndType(enterpriseId);
            return RestResponse.success();
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    
    /** @Description: 列表筛选支持配置
     * @author taogs
     * @Date 18:15 2020/3/11
     * @Param 
     * @return 
    */
    @RequestMapping("relevance-storefield")
    @ResponseBody
    public RestResponse setStoreField(String values){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        String key = "relevanceStorefield:"+enterpriseId;
        Object cache = RedisUtil.getCache(key);
        if(cache != null){
            //return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "请稍后重试");
        }
        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setEnterpriseId(1129);
        auditLogDTO.setApplyUserId(124);
        auditLogDTO.setProjectItemCode(Constants.PROJECT_ITEM_RISK_MODE_APPLY);
        JSONObject jsonObject = new JSONObject();
        // 自定义的信息
        jsonObject.put("enterpriseId", enterpriseId);
        jsonObject.put("values", values);
        jsonObject.put("userId", userId);
        auditLogDTO.setExtraInfo(jsonObject.toJSONString());
        auditLogApiService.apply(auditLogDTO);
        RedisUtil.setCache(key, 1, 12l, TimeUnit.HOURS);
        ServiceResponse serviceResponse = this.storeDictApiService.saveStoreField(enterpriseId, values, userId);
        if(serviceResponse.isSuccess()){
            LogUtils.createLog("自定义字段列表支持筛选配置", "自定义字段");
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

    @RequestMapping("list-storefield-relevance")
    @ResponseBody
    public RestResponse listStoreFieldRelevance(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<StoreDictVO> systemField = new ArrayList<>();
        systemField.add(new StoreDictVO(StoreESFieldsEnum.STORENAME.getDesc(), StoreESFieldsEnum.STORENAME.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.STORECODE.getDesc(), StoreESFieldsEnum.STORECODE.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.REGIONID.getDesc(), StoreESFieldsEnum.REGIONID.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.STOREGROUPID.getDesc(), StoreESFieldsEnum.STOREGROUPID.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.PROVINCES.getDesc(), StoreESFieldsEnum.PROVINCES.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.STOREBRANDS.getDesc(), StoreESFieldsEnum.STOREBRANDS.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.STATUS.getDesc(), StoreESFieldsEnum.STATUS.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.COMPLETESTATUS.getDesc(), StoreESFieldsEnum.COMPLETESTATUS.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.CREATETYPE.getDesc(), StoreESFieldsEnum.CREATETYPE.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.STORETYPE.getDesc(), StoreESFieldsEnum.STORETYPE.getField()));
        systemField.add(new StoreDictVO(StoreESFieldsEnum.ERPSTATUS.getDesc(), StoreESFieldsEnum.ERPSTATUS.getField()));
        ServiceResponse serviceResponse = this.storeDictApiService.listStoreFieldOfFilterText(enterpriseId);
        if(serviceResponse.isSuccess()){
            List<StoreDictVO> customFields = EntityUtil.changeEntityListByJSON(StoreDictVO.class, serviceResponse.getResult());
            systemField.addAll(customFields);
            return RestResponse.success(systemField);
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

}
