package com.gic.store.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.constant.StoreImportEnum;
import com.gic.store.dto.*;
import com.gic.store.service.*;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.utils.ExcelUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zhiwj
 * @date 2019/7/3
 */
@RestController
public class StoreImportController {
    private static final Logger logger = LogManager.getLogger(StoreImportController.class);

    @Autowired
    private StoreApiService storeApiService;

    @Autowired
    private StoreImportApiService storeImportApiService;

    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @Autowired
    private StoreDictApiService storeDictApiService;

    @Autowired
    private ProvincesApiService provincesApiService;

    @Autowired
    private StoreFieldApiService storeFieldApiService;

    @Autowired
    private StoreStrategyApiService storeStrategyApiService;

    /**
     * 下载模板
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping("/store-import-template-download")
    public Object download(HttpServletRequest request, HttpServletResponse response) throws Exception {
//        ClassPathResource res = new ClassPathResource("门店资料导入模板.xlsx");
        String regionIdStr = request.getParameter("regionId");
        if (StringUtils.isBlank(regionIdStr)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "请选择门店域");
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<List<String>> list = new ArrayList<>();

        ServiceResponse<List<List<String>>> titleServiceResponse = storeImportApiService.listImportTitle(enterpriseId, Integer.parseInt(regionIdStr));
        // 测试数据
        List<String> list3 = new ArrayList<>();
        Collections.addAll(list3, "1", "杭州测试总店", "A001", "自营", "上线", "12345678912", "杭州门店", "品牌1、品牌2", "浙江省", "杭州市", "西湖区", "华星时代广场", "经度", "纬度");
        Collections.addAll(list, list3);
        int size = list3.size();
        for (int i = 0; i < titleServiceResponse.getResult().get(0).size() - size; i++) {
            list3.add(" ");
        }
        Workbook workbook = ExcelUtils.getWorkbook("门店导入模板", titleServiceResponse.getResult().get(0), titleServiceResponse.getResult().get(1), list, regionIdStr);

        String fileName = "门店资料导入模板.xlsx";
        byte[] bytesName = fileName.getBytes("UTF-8");
        fileName = new String(bytesName, "ISO-8859-1");
        response.setHeader("content-disposition", "attachment;fileName=" + fileName);

        OutputStream out = response.getOutputStream();
        workbook.write(out);
        out.close();

        return RestResponse.success();
    }

    /**
     * 批量导入
     * @param request
     * @param file
     * @param pageQO
     * @return
     * @throws IOException
     */
    @RequestMapping("/store-tmp-file-upload")
    public RestResponse fileUpload(HttpServletRequest request, MultipartFile file, PageQO pageQO) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (file == null || file.isEmpty()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "文件不存在");
        }
        // 判断是否有正在导入的任务
        ServiceResponse<Boolean> importingStoreResponse = storeImportApiService.hasImportingStore(enterpriseId);
        if (importingStoreResponse.getResult()) {
            return RestResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "当前企业下有正在导入门店的任务，请稍后重试！");
        }
        if (file.getSize() > 0) {
            String fileName = file.getOriginalFilename();
            String uuId = request.getParameter("uuId");
            //获取配置信息，是否门店替换code
            String repeatStoreCodeConfig = request.getParameter("repeatStoreCodeConfig");
            if (StringUtils.isBlank(repeatStoreCodeConfig)) {
                repeatStoreCodeConfig = "0";
            }
            //如果门店code重复，是否替换更新
            boolean isReplace = "1".equals(repeatStoreCodeConfig);
            if (fileName.endsWith("xlsx") || fileName.endsWith("xls")) {
                try {
                    Workbook workbook = ExcelUtils.getWorkbookFromUpload(file.getInputStream(), fileName);
                    StoreRegionDTO storeRegion = this.readStoreRegion(enterpriseId, workbook);
                    if (storeRegion == null) {
                        return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店域不存在");
                    }
                    ServiceResponse<Map<Integer, Boolean>> isDateTypeMap = storeImportApiService.fieldIsDateType(enterpriseId, storeRegion.getRegionId());
                    List<Map<Integer, String>> data = ExcelUtils.readExcel(6, fileName.substring(fileName.lastIndexOf(".") + 1), workbook, isDateTypeMap.getResult());
                    ServiceResponse<List<List<String>>> titleServiceResponse = storeImportApiService.listImportTitle(enterpriseId, storeRegion.getRegionId());
                    List<String> titleList = titleServiceResponse.getResult().get(0);
                    if (CollectionUtils.isNotEmpty(data) && titleList.size() != data.get(0).size()) {
                        return RestResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "总列数与模板不一致");
                    }
                    //重置临时表数据
                    storeImportApiService.delStoreTmp(enterpriseId);
                    //批量插入数据到门店临时表，用于队列导入到门店表
                    for (Map<Integer, String> row : data) {
                        StoreTmpDTO bean = this.validateData(row, enterpriseId, storeRegion.getRegionId(), isReplace);
                        bean.setRegionId(storeRegion.getRegionId());
                        bean.setRegionName(storeRegion.getRegionName());
                        try {
                            bean.setSignKey(uuId);
                            bean.setOperatorId(UserDetailUtils.getUserDetail().getUserId());
                            bean.setOperatorName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
                            // 保存数据到临时表
                            storeImportApiService.save(bean);
                        } catch (Exception e) {
                            logger.warn(e);
                        }
                    }

                    if (storeImportApiService.hasImportingStore(enterpriseId).getResult()) {
                        // userId
                        Integer userId = UserDetailUtils.getUserDetail().getUserInfo().getUserId();
                        // 开队列
                        String r = storeImportApiService.importDataToStore(enterpriseId, userId.toString(), uuId, isReplace).getResult();
                        if (StringUtils.isNotBlank(r)) {
                            logger.warn("队列错误, {}", r);
                            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "网络错误");
                        }
                    } else {
                        Page page = storeImportApiService.listStoreTmp(enterpriseId, false, pageQO.getCurrentPage(), pageQO.getPageSize()).getResult();
                        Map<Object, Object> result = new HashMap<>(8);
                        result.put("errorData", page);
                        int errorRows = page.getTotalCount();
                        result.put("total", errorRows);
                        result.put("success", 0);
                        result.put("error", errorRows);
                        return RestResponse.success(result);
                    }
                } catch (Exception e) {
                    logger.warn("excel验证不通过", e);
                    return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "excel验证不通过");
                }
            } else {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "选择xlsx格式文件上传");
            }
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "选择xlsx格式文件上传");
        }
        LogUtils.createLog("导入门店", "门店");
        return RestResponse.success();
    }

    /**
     * 错误日志查看
     * @param page
     * @return
     */
    @RequestMapping("/store-import-load-error-data")
    public Object loadErrorData(PageQO page) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Map<String, Object> result = new HashMap<String, Object>();
        List<StoreTmpDTO> successList = storeImportApiService.listStoreTmp(enterpriseId, true).getResult();
        ServiceResponse<Page> serviceResponse = storeImportApiService.pageError(enterpriseId, page.getCurrentPage(), page.getPageSize());
        List<StoreTmpDTO> totalList = storeImportApiService.listAllStoreTmp(enterpriseId).getResult();
        Page pageResponse = serviceResponse.getResult();
        result.put("errorData", pageResponse);
        result.put("total", totalList == null ? 0 : totalList.size());
        result.put("success", successList == null ? 0 : successList.size());
        if (pageResponse.getResult().size() == 0) {
            result.put("error", 0);
        }else{
            // 减去标题行
            result.put("error", pageResponse.getResult().size() - 1);
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/get_store_batch_process")
    public Object getStoreBatchProcess(HttpServletRequest request, PageQO page) {

        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String uuId = request.getParameter("uuId");
//        String url = GlobalVar.ctxPropertiesMap.get("batch.task.url").replace("\\", "") + "/task-allocation/queryTask.json";
//        HashMap<String, String> param = new HashMap<>(4);
//        param.put("enterpriseId", enterpriseId.toString());
//        param.put("taskType", Constants.STORE_BATCH_IMPORT_TASK_TYPE.toString());
//        param.put("status", "1,2,3,4");
//        String result = HttpClientUtil.doPost(url, param);

        Map<String, Object> data = storeImportApiService.getStoreBatchProcess(enterpriseId, uuId, page.getCurrentPage(), page.getPageSize()).getResult();


//        Map<String, Object> data = new HashMap<String, Object>(8);
//        if (StringUtils.isNotBlank(result)) {
//            JSONObject resultObj = JSONObject.parseObject(result);
//            JSONArray jsonArr = JSONArray.parseArray(resultObj.getString("list"));
//            if (null != jsonArr) {
//                if (jsonArr.size() > 0) {
//                    for (int i = 0; i < jsonArr.size(); i++) {
//                        JSONObject jsonObj = jsonArr.getJSONObject(i);
//                        int taskTotal = jsonObj.getIntValue("taskTotal");
//                        int taskExecNum = jsonObj.getIntValue("taskExecNum");
//                        int taskFailNum = jsonObj.getIntValue("taskFailNum");
//                        String taskSignKey = jsonObj.getString("taskSignKey");
//                        if (uuId.equals(taskSignKey)) {
//                            data.put("taskTotal", taskTotal);
//                            data.put("taskExecNum", taskExecNum);
//                            data.put("taskFailNum", taskFailNum);
//                            if ((taskTotal - taskFailNum) > 0) {
//                                data.put("process", taskExecNum*100 / (taskTotal - taskFailNum));
//                                if (taskTotal == (taskExecNum + taskFailNum)) {
//                                    // 获取所有错误数据
//                                    Page resultPage = storeImportApiService.listStoreTmp(enterpriseId, false, page.getCurrentPage(), page.getPageSize()).getResult();
//                                    data.put("errorData", resultPage);
//                                }
//                            }
//                        }
//                    }
//                }
//            }
//        }
        return RestResponse.success(data);
    }

    /**
     * 导出错误日志
     * @param request
     * @param response
     * @return
     * @throws IOException
     */
    @RequestMapping("/export-error-data")
    public RestResponse exportErrorData(HttpServletRequest request, HttpServletResponse response) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String fileName = "门店导入错误记录.xlsx";
        List<StoreTmpDTO> listErrorData = storeImportApiService.listStoreTmp(enterpriseId, false).getResult();
        if (!listErrorData.isEmpty()) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
            fileName = sdf.format(listErrorData.get(0).getUpdateTime()) + fileName;
            List<List<String>> errorList = storeImportApiService.listError(enterpriseId).getResult();
            List<String> titleList = errorList.get(0);
            errorList.remove(0);
            Workbook workbook = ExcelUtils.getWorkbook("门店导入错误记录", titleList, null, errorList, null);
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);

            OutputStream out = response.getOutputStream();
            workbook.write(out);
            out.close();
            LogUtils.createLog("导出错误日志", "门店");
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "没有错误日志");
        }
    }

    private StoreTmpDTO validateData(Map<Integer, String> row, Integer enterpriseId, Integer regionId, boolean isNeedReplaceStoreCode) {
//        boolean validate = true;
        StoreTmpDTO bean = new StoreTmpDTO();
        bean.setEnterpriseId(enterpriseId);
        bean.setErrorMessage(StoreImportEnum.WAIT.getCode());

        StringBuilder errorMessage = new StringBuilder();
        // 验证门店名称
        String storeName = row.get(1);
        if (StringUtils.isBlank(storeName)) {
            errorMessage.append("门店名称不能为空,");
        } else {
            storeName = storeName.trim();
            bean.setStoreName(storeName);

            if (this.lengthValidate(storeName, 20)) {
                errorMessage.append("门店名称长度超过20个字,");
            }
            if (!isNeedReplaceStoreCode) {
                // 验证门店name唯一
                if (storeImportApiService.validateStoreName(enterpriseId, regionId, storeName).getResult()) {
                    errorMessage.append("门店名称已经存在,");
                }
            }
        }

        // 验证门店代码
        String storeCode = row.get(2);
        if (StringUtils.isBlank(storeCode)) {
            errorMessage.append("门店关联主键不能为空,");
        } else {
            storeCode = storeCode.trim();
            bean.setStoreCode(storeCode);

            if (this.lengthValidate(storeCode, 20)) {
                errorMessage.append("门店关联主键长度超过20个字,");
            }
            if (!isNeedReplaceStoreCode) {
                // 验证门店code唯一
                if (storeImportApiService.validateStoreCode(enterpriseId, regionId, storeCode).getResult()) {
                    errorMessage.append("门店编码重复,");
                }
            }
        }

        // 验证门店类型
        String storeType = row.get(3);
        if (StringUtils.isNotBlank(storeType)) {
            storeType = storeType.trim();
            bean.setStoreType(storeType);
            if (this.storeTypeValidate(enterpriseId, storeType) == -1) {
                errorMessage.append("门店类型不存在,");
            }
        }

        // 验证门店状态
        String erpStatus = row.get(4);
        if (StringUtils.isNotBlank(erpStatus)) {
            erpStatus = erpStatus.trim();
            bean.setErpStatus(erpStatus);
            if (this.erpStatusValidate(enterpriseId, erpStatus) == -1) {
                errorMessage.append("门店状态不存在,");
            }
        }

        // 验证门店电话
        String conactsPhone = row.get(5);
        bean.setConactsPhone(conactsPhone);
//        if (validate) {
//            if (StringUtils.isBlank(conactsPhone)) {
//                bean.setErrorMessage("门店电话不能为空");
//                validate = false;
//            }
//        }
        // 验证门店所属分组,
        String storeGroupName = row.get(6);
        if (StringUtils.isNotBlank(storeGroupName)) {
            storeGroupName = storeGroupName.trim();
            bean.setStoreGroupName(storeGroupName);
            // 填写分组后需要判断分组是否存在,不填写分组就策略
            if (StringUtils.isBlank(storeImportApiService.isExistStoreGroup(enterpriseId, storeGroupName).getResult())) {
                errorMessage.append("门店分组不存在,");
            }
        }


        // 验证品牌
        String brands = row.get(7);

        if (StringUtils.isBlank(brands)) {
            errorMessage.append("店招品牌不能为空,");
        } else {
            brands = brands.trim();
            bean.setBrands(brands);
            String errorMsg = brandsValidate(enterpriseId, brands);
            if (errorMsg != null) {
                errorMessage.append(errorMsg).append(",");
            }
        }

        // 验证省份
        String province = row.get(8);
        province = province == null ? null : province.trim();
        bean.setProvince(province);
//        if (validate) {
//            if (StringUtils.isBlank(province)) {
//                bean.setErrorMessage("门店省份不能为空");
//                validate = false;
//            }
//        }
        // 验证市
        String city = row.get(9);
        city = city == null ? null : city.trim();
        bean.setCity(city);
//        if (validate) {
//            if (StringUtils.isBlank(city)) {
//                bean.setErrorMessage("门店市不能为空");
//                validate = false;
//            }
//        }
        // 验证区
        String county = row.get(10);
        county = county == null ? null : county.trim();
        bean.setCounty(county);
//        if (validate) {
//            if (StringUtils.isBlank(county)) {
//                bean.setErrorMessage("门店区县不能为空");
//                validate = false;
//            }
//        }
        // 验证省市区数据的正确性
        if(StringUtils.isNotBlank(province) && StringUtils.isNotBlank(city) && StringUtils.isNotBlank(county)) {
            ServiceResponse<List<ProvinceDTO>> provinceResponse = provincesApiService.selectAllProvince();
            ServiceResponse<List<CityDTO>> cityResponse = provincesApiService.selectAllCity();
            ServiceResponse<List<CountyDTO>> countyResponse = provincesApiService.selectAllCounty();
            List<ProvinceDTO> provinceDTOList = provinceResponse.getResult();
            String finalProvince = province;
            long provinceCount = provinceDTOList.stream().filter(e -> StringUtils.equals(e.getProvinceName(), finalProvince)).count();
            List<CityDTO> cityDTOList = cityResponse.getResult();
            String finalCity = city;
            long cityCount = cityDTOList.stream().filter(e -> StringUtils.equals(e.getCityName(), finalCity)).count();
            List<CountyDTO> countyDTOList = countyResponse.getResult();
            String finalCounty = county;
            long countyCount = countyDTOList.stream().filter(e -> StringUtils.equals(e.getCountyName(), finalCounty)).count();
            if (countyCount == 0) {
                errorMessage.append("输入的区县信息有误,");
            }
            if (cityCount == 0) {
                errorMessage.append("输入的市信息有误,");
            }
            if (provinceCount == 0) {
                errorMessage.append("输入的省信息有误,");
            }
        }
        // 验证门店详细地址
        String address = row.get(11);
        bean.setAddress(address);
//        if (validate) {
//            if (StringUtils.isBlank(address)) {
//                bean.setErrorMessage("门店详细地址不能为空");
//                validate = false;
//            }
//        }

        bean.setLongitude(row.get(12));
        bean.setLatitude(row.get(13));

        // 经纬度校验
        if (StringUtils.isNotBlank(bean.getLongitude())) {
            try {
                Float longitude = Float.valueOf(bean.getLongitude());
                if (longitude < 0 || longitude > 180) {
                    errorMessage.append("经度错误,");
                }
            } catch (NumberFormatException e) {
                errorMessage.append("经度错误,");
            }
        }
        if (StringUtils.isNotBlank(bean.getLatitude())) {
            try {
                Float latitude = Float.valueOf(bean.getLatitude());
                if (latitude < 0 || latitude > 90) {
                    errorMessage.append("维度错误,");
                }
            } catch (NumberFormatException e) {
                errorMessage.append("维度错误,");
            }
        }

        // 自定义属性
        ServiceResponse<List<StoreFieldDTO>> fieldServiceResponse = storeFieldApiService.listStoreFieldByRegionId(regionId);
        List<StoreFieldDTO> fieldList = fieldServiceResponse.getResult();
        logger.info("门店域的ID：{}, 自定义字段数据:{}", regionId, JSON.toJSONString(fieldList));
        if (CollectionUtils.isNotEmpty(fieldList)) {
            List<StoreExtendDTO> extendDTOList = new ArrayList<>();
            for (int i = 0; i < fieldList.size(); i++) {
                StoreExtendDTO storeExtendDTO = new StoreExtendDTO();
                storeExtendDTO.setStoreFieldId(fieldList.get(i).getStoreFieldId());
                storeExtendDTO.setValue(row.get(14 + i));
                storeExtendDTO.setSort(fieldList.get(i).getSort());
                // 验证
                ServiceResponse fieldResponse = storeApiService.validateFieldExceptRepeat(storeExtendDTO.getValue(), fieldList.get(i));
                if (!fieldResponse.isSuccess()) {
                    errorMessage.append(fieldResponse.getMessage()).append(",");
                }
                extendDTOList.add(storeExtendDTO);
            }
            bean.setCustomField(JSON.toJSONString(extendDTOList));
        } else {
            bean.setCustomField(JSON.toJSONString(Collections.emptyList()));
        }

        if (errorMessage.length() > 0) {
            bean.setErrorMessage(errorMessage.deleteCharAt(errorMessage.length() - 1).toString());
        }
        return bean;
    }

    private String brandsValidate(Integer enterpriseId, String brands) {
        ServiceResponse<List<String>> response = storeImportApiService.listBrandByNotExist(enterpriseId, brands);
        if (response.isSuccess()) {
            List<String> brandList = response.getResult();
            if (CollectionUtils.isNotEmpty(brandList)) {
                String error = brandList.stream().reduce((x, y) -> x + " " + y).get();
                return "品牌 " + error + " 不存在";
            }
            return null;
        } else {
            return "系统错误";
        }
    }

    private int erpStatusValidate(Integer enterpriseId, String erpStatus) {
        ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreStatus(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<StoreDictDTO> list = serviceResponse.getResult();
            final String erpStatusTmp = erpStatus.replace("\\s+", "");
            long count = list.stream().filter(e -> e.getKey().equals(erpStatusTmp)).count();
            if (count > 0) {
                return (int) count;
            }
        }
        return -1;
    }

    private int storeTypeValidate(Integer enterpriseId, String storeType) {
        ServiceResponse<List<StoreDictDTO>> serviceResponse = storeDictApiService.listStoreType(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<StoreDictDTO> list = serviceResponse.getResult();
            final String storeTypeTmp = storeType.replace("\\s+", "");
            long count = list.stream().filter(e -> e.getKey().equals(storeTypeTmp)).count();
            if (count > 0) {
                return (int) count;
            }
        }
        return -1;
    }

    private boolean lengthValidate(String value, int len) {
        String _value = value.replaceAll("[^\\x00-\\xff]", "**");
        return _value.length() / 2.0 > (double) len;
    }

    private StoreRegionDTO readStoreRegion(Integer enterpriseId, Workbook workbook) {
        Sheet sheet = workbook.getSheetAt(0);
        Row row = sheet.getRow(0);
        Cell cell = row.getCell(0);
        cell.setCellType(Cell.CELL_TYPE_STRING);
        String value = cell.getStringCellValue();
        ServiceResponse<StoreRegionDTO> serviceResponse = storeRegionApiService.getStoreRegionByRegion(enterpriseId, Integer.valueOf(value));
        return serviceResponse.isSuccess() ? serviceResponse.getResult() : null;
    }

//    @RequestMapping("/test")
//    public Object re() {
//        List<StoreTmpDTO> storeTmpDTOS = storeImportApiService.listStoreTmp(1111, null, true);
//        System.out.println(storeTmpDTOS);
//        return null;
//    }
}
